/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal;

import static java.util.Collections.emptyList;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.introspection.IIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.IIntrospectionItem;
import org.fortiss.tooling.kernel.introspection.IIntrospectiveKernelService;
import org.fortiss.tooling.kernel.service.IKernelIntrospectionSystemService;
import org.fortiss.tooling.kernel.service.ITutorialService;
import org.fortiss.tooling.kernel.ui.extension.IContextMenuContributor;
import org.fortiss.tooling.kernel.ui.extension.IContextMenuMultiSelectionContributor;
import org.fortiss.tooling.kernel.ui.extension.data.ContextMenuContextProvider;
import org.fortiss.tooling.kernel.ui.introspection.items.ContextMenuKISSDetailsItem;
import org.fortiss.tooling.kernel.ui.service.IActionService;
import org.fortiss.tooling.kernel.ui.service.IContextMenuService;
import org.fortiss.tooling.kernel.ui.service.ITutorialUIService;
import org.fortiss.tooling.kernel.utils.ExtensionPointUtils;
import org.fortiss.tooling.kernel.utils.LoggingUtils;
import org.osgi.framework.Bundle;

/**
 * This class implements the {@link IContextMenuService} interface.
 * 
 * @author hoelzl
 */
public class ContextMenuService implements IContextMenuService, IIntrospectiveKernelService {
	/** The singleton service instance. */
	private static ContextMenuService INSTANCE = new ContextMenuService();

	/** Returns the service instance. */
	public static ContextMenuService getInstance() {
		return INSTANCE;
	}

	/** The model element handler extension point ID. */
	private static final String EXTENSION_POINT_NAME =
			"org.fortiss.tooling.kernel.ui.contextMenuContribution";

	/** The model element handler configuration element name. */
	private static final String CONFIGURATION_ELEMENT_NAME = "contextMenuContribution";

	/** The model element handler class attribute name. */
	private static final String HANDLER_CLASS_ATTRIBUTE_NAME = "contributor";

	/** Stores the registered context menu contributors. */
	private final List<IContextMenuContributor> contextMenuContributorList =
			new ArrayList<IContextMenuContributor>();

	/** Initializes the service. */
	public void initializeService() {
		// nothing to do here
	}

	/** Starts the service. */
	public void startService() {
		setupContextMenuContributors();
		IKernelIntrospectionSystemService.getInstance().registerService(this);
	}

	/** Registers the given context menu contributor with the service. */
	@Override
	public void registerContextMenuContributor(IContextMenuContributor contributor) {
		contextMenuContributorList.add(contributor);
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionDescription() {
		return getIntrospectionLabel() +
				"\n\nThis service allows the registration of context menu items, which are shown by right-clicking" +
				"\nin the navigator tree view. This service replaces the heavyweight mechanism of Eclipse and" +
				"\nprovides context menu items directly related to a EObject selection." +
				"\nEach context menu item can be assigned to a menu section, thus allowing a coarse ordering of items." +
				"\n\nThe service extension point is '" + EXTENSION_POINT_NAME + "'.";
	}

	/** {@inheritDoc} */
	@Override
	public MenuManager createDefaultContextMenu(ContextMenuContextProvider contextProvider) {
		MenuManager menuManager = new MenuManager("#PopupMenu");
		menuManager.setRemoveAllWhenShown(true);
		menuManager.addMenuListener(new MenuCreator(contextProvider));

		return menuManager;
	}

	/** Initializes the context menu contributors from plugin extensions. */
	private void setupContextMenuContributors() {
		for(IConfigurationElement ce : ExtensionPointUtils
				.getConfigurationElements(EXTENSION_POINT_NAME, CONFIGURATION_ELEMENT_NAME)) {
			Bundle bundle = ExtensionPointUtils.getBundle(ce);
			try {
				Class<?> handlerClass = ExtensionPointUtils
						.loadClass(ce.getAttribute(HANDLER_CLASS_ATTRIBUTE_NAME), bundle);

				IContextMenuContributor contextMenuContributor =
						(IContextMenuContributor)handlerClass.getConstructor().newInstance();
				contextMenuContributorList.add(contextMenuContributor);
			} catch(Exception ex) {
				LoggingUtils.error(ToolingKernelActivator.getDefault(), ex.getMessage(), ex);
			}
		}
	}

	/** Wrapper class for menu creation. */
	private class MenuCreator implements IMenuListener {

		/** Stores the context object. */
		private final ContextMenuContextProvider contextProvider;

		/** Constructor. */
		public MenuCreator(ContextMenuContextProvider contextProvider) {
			this.contextProvider = contextProvider;
		}

		/** {@inheritDoc} */
		@Override
		public void menuAboutToShow(IMenuManager menu) {
			if(!menu.isEmpty()) {
				return;
			}
			addVisibleSectionSeparator(menu, TOP_MOST_MENU_SECTION_ID);
			addVisibleSectionSeparator(menu, BEFORE_GLOBAL_MENU_SECTION_ID);
			addVisibleSectionSeparator(menu, ActionService.GLOBAL_DEFAULT_MENU_SECTION_ID);
			addVisibleSectionSeparator(menu, AFTER_GLOBAL_MENU_SECTION_ID);
			addVisibleSectionSeparator(menu, REPOSITORY_MENU_SECTION_ID);
			addVisibleSectionSeparator(menu, IWorkbenchActionConstants.MB_ADDITIONS);
			addVisibleSectionSeparator(menu, BOTTOM_MOST_MENU_SECTION_ID);

			addContributions(menu, contextProvider);

			if(!ITutorialService.getInstance().isTutorialActive()) {
				IActionService.getInstance().addGlobalDefaultActionSectionToMenu(menu);
			} else {

				IActionService.getInstance().addGlobalEditingActionsToMenu(menu,
						ITutorialUIService.getInstance().globalCopyPasteActionsVisible(),
						ITutorialUIService.getInstance().globalDeleteActionVisible(),
						ITutorialUIService.getInstance().globalRenameActionVisible(),
						ITutorialUIService.getInstance().globalSelectAllActionVisible());
			}
		}
	}

	/** Adds registered contributions to the given menu. */
	private void addContributions(IMenuManager menu, ContextMenuContextProvider contextProvider) {
		List<EObject> selection = contextProvider.getSelectedModelElementList();
		if(selection == null) {
			// do nothing
		} else if(selection.size() == 1) {
			singleSelectionElementContributions(menu, contextProvider);
		} else if(selection.size() > 1) {
			addMultiSelectionContributions(menu, contextProvider, selection);
		}
	}

	/** Populates the menu with contributions to a single model element selection. */
	private void singleSelectionElementContributions(IMenuManager menu,
			ContextMenuContextProvider contextProvider) {
		EObject selectionElem = contextProvider.getSelectedModelElement();
		for(IContextMenuContributor contributor : contextMenuContributorList) {
			String menuSectionID = contributor.getMenuSectionID();
			if(menuSectionID == null) {
				menuSectionID = IWorkbenchActionConstants.MB_ADDITIONS;
			}

			// get contributions from contributor
			List<IContributionItem> items =
					contributor.getContributedItems(selectionElem, contextProvider);
			if(items == null) {
				continue;
			}

			// active tutorial may filter contributions
			if(ITutorialService.getInstance().isTutorialActive()) {
				ITutorialUIService service = ITutorialUIService.getInstance();
				Predicate<? super IContributionItem> isVisible =
						i -> service.contextMenuContributionVisible(selectionElem, i);
				items = items.stream().filter(isVisible).collect(Collectors.toList());
			}
			if(items == null) {
				continue;
			}

			// populate menu
			for(IContributionItem item : items) {
				menu.appendToGroup(menuSectionID, item);
			}
		}
	}

	/** Populates the menu with contributions to multiple model element selection. */
	private void addMultiSelectionContributions(IMenuManager menu,
			ContextMenuContextProvider contextProvider, List<EObject> selection) {
		for(IContextMenuContributor contributor : contextMenuContributorList) {
			if(contributor instanceof IContextMenuMultiSelectionContributor) {
				IContextMenuMultiSelectionContributor multicontributor =
						(IContextMenuMultiSelectionContributor)contributor;
				String menuSectionID = contributor.getMenuSectionID();
				if(menuSectionID == null) {
					menuSectionID = IWorkbenchActionConstants.MB_ADDITIONS;
				}

				// get contributions from contributor
				List<IContributionItem> items =
						multicontributor.getContributedItems(selection, contextProvider);
				if(items == null) {
					continue;
				}

				// active tutorial does not support multiple selections
				if(ITutorialService.getInstance().isTutorialActive()) {
					continue;
				}

				// populate menu
				for(IContributionItem item : items) {
					menu.appendToGroup(menuSectionID, item);
				}
			}
		}
	}

	/** Adds a visible section separator. */
	private void addVisibleSectionSeparator(IMenuManager menu, String sectionName) {
		Separator separator = new Separator(sectionName);
		separator.setVisible(true);
		menu.add(separator);
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionLabel() {
		return "Context Menu Service";
	}

	/** {@inheritDoc} */
	@Override
	public List<IIntrospectionItem> getIntrospectionItems() {
		return emptyList();
	}

	/** {@inheritDoc} */
	@Override
	public boolean showInIntrospectionNavigation() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public IIntrospectionDetailsItem getDetailsItem() {
		return new ContextMenuKISSDetailsItem(contextMenuContributorList);
	}

	/** {@inheritDoc} */
	@Override
	public List<IContributionItem> getContextMenuContributions(EObject element,
			ContextMenuContextProvider contextProvider) {
		List<IContributionItem> ret = new ArrayList<IContributionItem>();

		// Iterate all ContextMenuContributors to collect menu items.
		for(IContextMenuContributor contributor : contextMenuContributorList) {
			List<IContributionItem> contributions =
					contributor.getContributedItems(element, contextProvider);
			if(contributions != null) {
				ret.addAll(contributions);
			}
		}

		return ret;
	}
}
